import { Persistent } from '../storages/persistent.js';
import { defineConfig } from '@agile-ts/utils';
import { getStorageManager } from '../storages/index.js';
import { LogCodeManager } from '../logCodeManager.js';

const _CollectionPersistent = class extends Persistent {
  constructor(collection, config = {}) {
    super(collection.agileInstance(), {
      loadValue: false
    });
    config = defineConfig(config, {
      loadValue: true,
      storageKeys: [],
      defaultStorageKey: null
    });
    this.collection = () => collection;
    this.instantiatePersistent({
      key: config.key,
      storageKeys: config.storageKeys,
      defaultStorageKey: config.defaultStorageKey
    });
    if (this.ready && config.loadValue)
      this.initialLoading();
  }
  async initialLoading() {
    super.initialLoading().then(() => {
      this.collection().isPersisted = true;
    });
  }
  async loadPersistedValue(storageItemKey) {
    if (!this.ready)
      return false;
    const _storageItemKey = storageItemKey ?? this._key;
    const isPersisted = await getStorageManager()?.get(_storageItemKey, this.config.defaultStorageKey);
    if (!isPersisted)
      return false;
    const loadValuesIntoCollection = async () => {
      const defaultGroup = this.collection().getDefaultGroup();
      if (defaultGroup == null)
        return false;
      const defaultGroupStorageKey = _CollectionPersistent.getGroupStorageKey(defaultGroup._key, _storageItemKey);
      defaultGroup.loadedInitialValue = false;
      defaultGroup.persist(defaultGroupStorageKey, {
        loadValue: false,
        defaultStorageKey: this.config.defaultStorageKey || void 0,
        storageKeys: this.storageKeys,
        followCollectionPersistKeyPattern: false
      });
      if (defaultGroup.persistent?.ready)
        await defaultGroup.persistent.initialLoading();
      for (const itemKey of defaultGroup._value) {
        const item = this.collection().getItem(itemKey);
        const itemStorageKey = _CollectionPersistent.getItemStorageKey(itemKey, _storageItemKey);
        if (item != null) {
          item.persist(itemStorageKey, {
            loadValue: false,
            defaultStorageKey: this.config.defaultStorageKey || void 0,
            storageKeys: this.storageKeys,
            followCollectionPersistKeyPattern: false
          });
          if (item?.persistent?.ready)
            await item.persistent.initialLoading();
        } else {
          const placeholderItem = this.collection().getItemWithReference(itemKey);
          placeholderItem?.persist(itemStorageKey, {
            loadValue: false,
            defaultStorageKey: this.config.defaultStorageKey,
            storageKeys: this.storageKeys,
            followCollectionPersistKeyPattern: false
          });
          if (placeholderItem?.persistent?.ready) {
            const loadedPersistedValueIntoItem = await placeholderItem.persistent.loadPersistedValue();
            if (loadedPersistedValueIntoItem) {
              this.collection().assignItem(placeholderItem, {
                overwrite: true,
                rebuildGroups: false
              });
              placeholderItem.isPersisted = true;
              this.collection().size += 1;
            }
          }
        }
      }
      defaultGroup.loadedInitialValue = true;
      return true;
    };
    const success = await loadValuesIntoCollection();
    if (success)
      this.setupSideEffects(_storageItemKey);
    return success;
  }
  async persistValue(storageItemKey) {
    if (!this.ready)
      return false;
    const _storageItemKey = storageItemKey ?? this._key;
    const defaultGroup = this.collection().getDefaultGroup();
    if (defaultGroup == null)
      return false;
    const defaultGroupStorageKey = _CollectionPersistent.getGroupStorageKey(defaultGroup._key, _storageItemKey);
    getStorageManager()?.set(_storageItemKey, true, this.storageKeys);
    defaultGroup.persist(defaultGroupStorageKey, {
      defaultStorageKey: this.config.defaultStorageKey || void 0,
      storageKeys: this.storageKeys,
      followCollectionPersistKeyPattern: false
    });
    for (const itemKey of defaultGroup._value) {
      const item = this.collection().getItem(itemKey);
      const itemStorageKey = _CollectionPersistent.getItemStorageKey(itemKey, _storageItemKey);
      item?.persist(itemStorageKey, {
        defaultStorageKey: this.config.defaultStorageKey || void 0,
        storageKeys: this.storageKeys,
        followCollectionPersistKeyPattern: false
      });
    }
    this.setupSideEffects(_storageItemKey);
    this.isPersisted = true;
    return true;
  }
  setupSideEffects(storageItemKey) {
    const _storageItemKey = storageItemKey ?? this._key;
    const defaultGroup = this.collection().getDefaultGroup();
    if (defaultGroup == null)
      return;
    defaultGroup.addSideEffect(_CollectionPersistent.defaultGroupSideEffectKey, (instance) => this.rebuildStorageSideEffect(instance, _storageItemKey), { weight: 0 });
  }
  async removePersistedValue(storageItemKey) {
    if (!this.ready)
      return false;
    const _storageItemKey = storageItemKey ?? this._key;
    const defaultGroup = this.collection().getDefaultGroup();
    if (!defaultGroup)
      return false;
    const defaultGroupStorageKey = _CollectionPersistent.getGroupStorageKey(defaultGroup._key, _storageItemKey);
    getStorageManager()?.remove(_storageItemKey, this.storageKeys);
    defaultGroup.persistent?.removePersistedValue(defaultGroupStorageKey);
    defaultGroup.removeSideEffect(_CollectionPersistent.defaultGroupSideEffectKey);
    for (const itemKey of defaultGroup._value) {
      const item = this.collection().getItem(itemKey);
      const itemStorageKey = _CollectionPersistent.getItemStorageKey(itemKey, _storageItemKey);
      item?.persistent?.removePersistedValue(itemStorageKey);
    }
    this.isPersisted = false;
    return true;
  }
  formatKey(key) {
    if (key == null && this.collection()._key)
      return this.collection()._key;
    if (key == null)
      return;
    if (this.collection()._key == null)
      this.collection()._key = key;
    return key;
  }
  rebuildStorageSideEffect(group, storageItemKey) {
    const collection = group.collection();
    const _storageItemKey = storageItemKey || collection.persistent?._key;
    if (group.previousStateValue.length === group._value.length)
      return;
    const addedKeys = group._value.filter((key) => !group.previousStateValue.includes(key));
    const removedKeys = group.previousStateValue.filter((key) => !group._value.includes(key));
    addedKeys.forEach((itemKey) => {
      const item = collection.getItem(itemKey);
      const itemStorageKey = _CollectionPersistent.getItemStorageKey(itemKey, _storageItemKey);
      if (item != null && !item.isPersisted)
        item.persist(itemStorageKey, {
          defaultStorageKey: this.config.defaultStorageKey || void 0,
          storageKeys: this.storageKeys,
          followCollectionPersistKeyPattern: false
        });
    });
    removedKeys.forEach((itemKey) => {
      const item = collection.getItem(itemKey);
      const itemStorageKey = _CollectionPersistent.getItemStorageKey(itemKey, _storageItemKey);
      if (item != null && item.isPersisted)
        item.persistent?.removePersistedValue(itemStorageKey);
    });
  }
  static getItemStorageKey(itemKey, collectionKey) {
    if (itemKey == null || collectionKey == null)
      LogCodeManager.log("1A:02:00");
    if (itemKey == null)
      itemKey = "unknown";
    if (collectionKey == null)
      collectionKey = "unknown";
    return this.storageItemKeyPattern.replace("${collectionKey}", collectionKey.toString()).replace("${itemKey}", itemKey.toString());
  }
  static getGroupStorageKey(groupKey, collectionKey) {
    if (groupKey == null || collectionKey == null)
      LogCodeManager.log("1A:02:01");
    if (groupKey == null)
      groupKey = "unknown";
    if (collectionKey == null)
      collectionKey = "unknown";
    return this.storageGroupKeyPattern.replace("${collectionKey}", collectionKey.toString()).replace("${groupKey}", groupKey.toString());
  }
};
let CollectionPersistent = _CollectionPersistent;
CollectionPersistent.defaultGroupSideEffectKey = "rebuildGroupStorageValue";
CollectionPersistent.storageItemKeyPattern = "_${collectionKey}_item_${itemKey}";
CollectionPersistent.storageGroupKeyPattern = "_${collectionKey}_group_${groupKey}";

export { CollectionPersistent };
