import { shared as sharedAgileInstance } from './shared.js';
import { Agile } from './agile.js';
import { LogCodeManager } from './logCodeManager.js';
import { Observer } from './runtime/observer.js';
import { isFunction, normalizeArray } from '@agile-ts/utils';

function getAgileInstance(instance) {
  try {
    if (instance != null) {
      const _agileInstance = isFunction(instance["agileInstance"]) ? instance["agileInstance"]() : instance["agileInstance"];
      if (_agileInstance)
        return _agileInstance;
    }
    if (sharedAgileInstance instanceof Agile) {
      return sharedAgileInstance;
    }
    return globalThis[Agile.globalKey];
  } catch (e) {
    LogCodeManager.log("20:03:00", [], instance);
  }
  return void 0;
}
function extractObservers(instances) {
  const observers = [];
  const tempInstancesArray = normalizeArray(instances, {
    createUndefinedArray: true
  });
  for (const instance of tempInstancesArray) {
    if (instance == null) {
      observers.push({});
      continue;
    }
    if (instance.isCollection) {
      observers.push(instance.getGroupWithReference(instance.config.defaultGroupKey).observers);
      continue;
    }
    if (instance["observer"] && instance["observer"] instanceof Observer) {
      observers.push({ value: instance["observer"] });
      continue;
    }
    if (instance["observers"]) {
      const extractedObservers = {};
      for (const key in instance["observers"]) {
        if (instance["observers"][key] instanceof Observer) {
          extractedObservers[key] = instance["observers"][key];
        }
      }
      observers.push(extractedObservers);
      continue;
    }
    if (instance instanceof Observer) {
      observers.push({ value: instance });
      continue;
    }
    observers.push({});
  }
  return Array.isArray(instances) ? observers : observers[0];
}
function extractRelevantObservers(instances, observerType) {
  const depsWithIndicator = {};
  const depsWithNoIndicator = [];
  for (const depKey in instances) {
    const extractedObservers = extractObservers(instances[depKey]);
    let observer = void 0;
    if (observerType != null && extractedObservers[observerType] != null)
      observer = extractedObservers[observerType];
    if (observerType == null)
      observer = extractedObservers["output"] ?? extractedObservers["value"];
    if (Array.isArray(instances))
      depsWithNoIndicator.push(observer);
    else
      depsWithIndicator[depKey] = observer;
  }
  return Array.isArray(instances) ? depsWithNoIndicator : depsWithIndicator;
}
function optionalRequire(moduleName, error = true) {
  let requiredPackage = null;
  try {
    requiredPackage = require(moduleName);
  } catch (e) {
    if (error) {
      LogCodeManager.log("20:03:02", [moduleName]);
      console.log(e);
    }
  }
  return requiredPackage;
}
function globalBind(key, instance, overwrite = false) {
  try {
    if (overwrite) {
      globalThis[key] = instance;
      return true;
    }
    if (globalThis[key] == null) {
      globalThis[key] = instance;
      return true;
    }
  } catch (e) {
    LogCodeManager.log("20:03:01", [key]);
  }
  return false;
}
const runsOnServer = () => {
  return !(typeof window !== "undefined" && typeof window.document !== "undefined" && typeof window.document.createElement !== "undefined");
};

export { extractObservers, extractRelevantObservers, getAgileInstance, globalBind, optionalRequire, runsOnServer };
